% spnhpoc_detectaxes
% 
% Input:
%   1. Folder path containing the board meshes.
% Output:
% 
%   1. Compute princpal vector of each of the 2 patches.
%   2. Visualize them.
%   3. Calculate the mean orientation.
%   4. Steer all the meshes according to the mean orientation.
%   5. Calculate the std, plot the range of angulation deviation.
% Based on 
function main(pre)
%%
% addpath 'D:\tch\ml\toolbox_graph'
% addpath 'D:\tch\ml\colortoolbox_v1_1'

%%
show_vermsg();

%%
fprintf('\nLooking for files in path %s ...\n', pre);
name_list = dir([pre '\*.obj']);
NNames = length(name_list);
fprintf('There are %d files found.\n', NNames);
if (NNames == 0)
    disp('End');
    return
end

r=1;
disp('To be processed:');
for k=1:NNames
    cur_name = name_list(k).name;
    name_obj = [pre '\' cur_name];
    
   	fid = fopen(name_obj,'rt');
	if (fid == -1)
		disp(['Skipping ', cur_name]);
		continue;
	end
    fprintf('Mesh #%d\t %s\n', k, cur_name);
    names2{r}=cur_name;
    r=r+1;
end

% % disp('Avaliable meshes:');
% % disp(names2);

fclose all;
NObs = r-1;

%%
% if matlabpool('size') == 0 % checking to see if my pool is already open
%     disp('Creating parpool 5...');
%     matlabpool open 5
% end

%%
All_HuvecO = zeros(NObs, 3);
All_VuvecO = zeros(NObs, 3);
All_HuvecFXY = zeros(NObs, 3);
All_VuvecFXY = zeros(NObs, 3);
All_HuvecF1 = zeros(NObs, 3);
All_VuvecF1 = zeros(NObs, 3);

range_avg = 1:NObs;

figno = -1; figno4 = -1;
%% No need edit below
figno=figure; hold on; grid on; xlabel('x');ylabel('y');zlabel('z'); title('extracted patches');
% figno2=figure;hold on; grid on; xlabel('x');ylabel('y');zlabel('z'); title('Fit to XY plane');
figno4=figure;hold on; grid on; xlabel('x');ylabel('y');zlabel('z'); title('Fitted');


% parfor k=1:NObs
for k=1:NObs
    % for k=1:52  
    disp(['Processing mesh #', num2str(k), '.']);
    cur_name = names2{k};
    [pathstr,name,ext] = fileparts(cur_name) ;
    name_obj = [pre '\' cur_name];
    name_tex = [pre '\' name '.jpg'];
    
%     disp(['Loading ', name, '...']);
    
    [vertex_h, vertex_v] = extract_patches_obj(name_obj, name_tex, figno);
    [Vuvec, Huvec] = compute_1PRYsq(vertex_h, vertex_v, figno);
    
%     disp('Orientation of the capture');
    direction = -2*((Huvec(:,1)>0)-0.5)*ones(1,3);
    a_Huvec = acosd(Huvec.*direction);
    direction = -2*((Vuvec(:,2)>0)-0.5)*ones(1,3);
    a_Vuvec = acosd(Vuvec.*direction);
    
%     disp('Fit onto XY plane');
    Xvec = Huvec .* (2*((Huvec(:,1)>0)-0.5)*ones(1,3));
    Yvec = Vuvec .* (2*((Vuvec(:,2)>0)-0.5)*ones(1,3));
    [Vuvec2, Huvec2, vv, vh, mean_vall] = compute_1PRYsq_trans3(vertex_h, vertex_v, Xvec, Yvec, figno4);
%     disp('Orientation of fitted onto XY plane');
    direction = -2*((Huvec2(:,1)>0)-0.5)*ones(1,3);
    a_Huvec2 = acosd(Huvec2.*direction);
    direction = -2*((Vuvec2(:,2)>0)-0.5)*ones(1,3);
    a_Vuvec2 = acosd(Vuvec2.*direction);
    
    All_meanV(k,:) = mean_vall;
    All_HuvecO(k,:) = Huvec;
    All_VuvecO(k,:) = Vuvec;
    All_HuvecFXY(k,:) = Huvec2;
    All_VuvecFXY(k,:) = Vuvec2;
  
end

%%
%%%%%%%%%%%% Mean orientation vectors
% Correct the possible flipping respectively.
% V: y should be same sign.
% H: x should be same sign.
direction = 2*((All_HuvecO(range_avg,1)>0)-0.5)*ones(1,3);
HuvecS = mean(All_HuvecO(range_avg,:).*direction)
direction = 2*((All_VuvecO(range_avg,2)>0)-0.5)*ones(1,3);
VuvecS = mean(All_VuvecO(range_avg,:).*direction)

timenow = datestr(now,'HHMMSS');
fname_axes = [pre '\' 'Axes-' date '-' timenow '.mat']
disp(['Save the ref axes as HuvecS, VuvecS in ' fname_axes  ]);
save(fname_axes , 'HuvecS', 'VuvecS');

[err_HO, err_VO, mean_HO, mean_VO] = cal_angulations_stat(All_HuvecO(range_avg,:), All_VuvecO(range_avg,:), 'Orig');

figure
hold on;
grid on;
plot(range_avg, err_VO(:,1),'r^');
plot(range_avg, err_VO(:,2),'r+');
plot(range_avg, err_HO(:,1),'ro');
legend('FXY Pitch #1', 'FXY Roll VY #1', 'FXY Yaw #1');
title('Deviation to XYZ axes')
xlabel('Trail')
ylabel('Degree (^o)');

%%
% matlabpool close

exitcode = 0;
return



function show_vermsg()
fprintf('\nSP-NHP orientation calibration\nDetect axes from reference board meshes (for DI3D)\n\n');
fprintf('Author: Richard Tai-Chiu Hsung\n');
fprintf('Faculty of Dentistry (OMFS), The University of Hong Kong\n');
fprintf('email: tchsung@hku.hk\n');
fprintf('Last update: Oct 17, 2014\n');
return