% spnhpoc_correctmesh
% 
% Input:
%   1. Folder path containing the board meshes.
% Output:
% 
%   1. Compute princpal vector of each of the 2 patches.
%   2. Visualize them.
%   3. Calculate the mean orientation.
%   4. Steer all the meshes according to the mean orientation.
%   5. Calculate the std, plot the range of angulation deviation.
% Based on 
function main_cm(pre)
%%
% addpath 'D:\tch\ml\toolbox_graph'
% addpath 'D:\tch\ml\colortoolbox_v1_1'

%%
show_vermsg();

%%
fprintf('\nLooking for axes file in path %s ...\n', pre);
name_list = dir([pre '\*.mat']);
NNames = length(name_list);
if (NNames == 0)
    fprintf('No axes file found.\n\n');
    return
end
latest_dn = name_list(1).datenum;
latest_n  = 1;
disp(name_list(1).name);
for n=2:NNames
    if (name_list(n).datenum > latest_dn)
        disp(name_list(n).name);
        latest_dn = name_list(n).datenum;
        latest_n = n;
    end
end
fname_axes = [ pre '\' name_list(latest_n).name];
fprintf('Select the latest one %d: %s\n', latest_n, fname_axes);
load(fname_axes);

%%
fprintf('\nLooking for files in path %s ...\n', pre);
name_list = dir([pre '\*.obj']);
NNames = length(name_list);
fprintf('There are %d files found.\n', NNames);
if (NNames == 0)
    disp('End');
    return
end

r=1;
disp('To be processed:');
for k=1:NNames
    cur_name = name_list(k).name;
    name_obj = [pre '\' cur_name];
    
   	fid = fopen(name_obj,'rt');
	if (fid == -1)
		disp(['Skipping ', cur_name]);
		continue;
	end
    fprintf('Mesh #%d\t %s\n', k, cur_name);
    names2{r}=cur_name;
    r=r+1;
end

% % disp('Avaliable meshes:');
% % disp(names2);

fclose all;
NObs = r-1;

%%
% if matlabpool('size') == 0 % checking to see if my pool is already open
%     disp('Creating parpool 5...');
%     matlabpool open 5
% end

%%
figno = -1;
for k=1:NObs
    disp(['Processing mesh #', num2str(k), '.']);
    cur_name = names2{k};
    [pathstr,name,ext] = fileparts(cur_name) ;
    name_obj = [pre '\' cur_name];
%     name_tex = [pre '\' name '.jpg'];
    name_objnhp = [name '_nhp.obj'];
    name_output = [pre '\' name_objnhp];
    
    disp(['Loading ', cur_name, '...']);

    [v,faces,vertexnormal,texcorrd, texX, texY, ftexi, fnmi, para_g, para_usemtl] = read_obj2(name_obj);
    [v2, mean_vall] = trans3_with_XYaxes(v, HuvecS, VuvecS, figno);

	fprintf('Output %s\n', name_output);
	write_obj2(name_output, v2, vertexnormal, texcorrd, faces, ftexi, fnmi, name, para_usemtl); % for di3d
end


fprintf('Done\n\n');
exitcode = 0;
return



function show_vermsg()
fprintf('\nSP-NHP orientation calibration\nCorrect mesh\n\n');
fprintf('\nspnhpoc_correctmesh input_folder\n(The most recent will be used)\n');
fprintf('Author: Richard Tai-Chiu Hsung\n');
fprintf('Faculty of Dentistry (OMFS), The University of Hong Kong\n');
fprintf('email: tchsung@hku.hk\n');
fprintf('Last update: Nov 12, 2014\n');
return