function [vertex,faces,vertexnormal,texcorrd, texX, texY, ftexi, fnmi, para_g, para_usemtl] = read_obj2(filename)

% read_obj - load a .obj file.
%
%   [vertex,face,normal] = read_obj(filename);
%
%   faces    : list of facesangle elements
%   vertex  : node vertexinatates
%   normal : normal vector list <-- Not supported
%
%   Copyright (c) 2008 Gabriel Peyre
% 
% Modified by Richard Hsung
% Nov 4, 2013.
% 

fid = fopen(filename);
if fid<0
    error(['Cannot open ' filename '.']);
end
disp(['Reading>>' filename])

frewind(fid);
a = fscanf(fid,'%c',1);
if strcmp(a, 'P')
    % This is the montreal neurological institute (MNI) specific ASCII facesangular mesh data structure.
    % For FreeSurfer software, a slightly different data input coding is
    % needed. It will be provided upon request.
    fscanf(fid,'%f',5);
    n_points=fscanf(fid,'%i',1);
    vertex=fscanf(fid,'%f',[3,n_points]);
    normal=fscanf(fid,'%f',[3,n_points]);
    n_faces=fscanf(fid,'%i',1);
    fscanf(fid,'%i',5+n_faces);
    faces=fscanf(fid,'%i',[3,n_faces])'+1;
    fclose(fid);
    return;
end

frewind(fid);
vertex = [];
vertexnormal = [];
faces = [];
ftexi = [];
fnmi = [];
texcorrd = [];
para_g = [];
para_usemtl = [];
while 1
    s = fgetl(fid);
    if ~ischar(s), 
        break;
    end
    if (~isempty(s))
        len_s = length(s);
        if strcmp(s(1), 'f')
            % face
            fv = sscanf(s,'f %d/%d/%d %d/%d/%d %d/%d/%d');
            if (length(fv) == 9)
                f_vi = fv([1 4 7]);
                f_vti = fv([2 5 8]);
                f_vni = fv([3 6 9]);
                faces(:,end+1) = f_vi;
                ftexi(:,end+1) = f_vti;
                fnmi(:,end+1) = f_vni;
            else
                fv = sscanf(s,'f %d//%d %d//%d %d//%d');
                f_vi = fv([1 3 5]);
                f_vni = fv([2 4 6]);
                faces(:,end+1) = f_vi;
                fnmi(:,end+1) = f_vni;
            end
        elseif len_s > 3
            if strcmp(s(1:2), 'v ')
                % vertex
                vertex0 = sscanf(s(3:end), '%f %f %f');
                vertex(:,end+1) = vertex0(1:3);
            elseif strcmp(s(1:2), 'vt')
                % Texture coordinate
        %     ['>>' s '<<']
                texcorrd(:,end+1) = sscanf(s(4:end), '%f %f');
            elseif strcmp(s(1:2), 'vn')
                % Texture coordinate
        %     ['>>' s '<<']
                vertexnormal(:,end+1) = sscanf(s(4:end), '%f %f %f');
            elseif strcmp(s(1:2), 'g ')
                para_g = s(3:end);
            end
            if len_s > 7
                if strcmp(s(1:7), 'usemtl ')
                    para_usemtl = s(8:end);
                end
            end
        end
    end
end
vertex=vertex';
nf = length(faces);
nv = length(vertex);
% nvt = length(texcorrd);

% Map the texture to each vertex.
texX = zeros(nv,1);
texY = zeros(nv,1);
if (length(ftexi)>1) 
    texX(reshape(faces,1,[])) = texcorrd(1,reshape(ftexi,1,[]));
    texY(reshape(faces,1,[])) = texcorrd(2,reshape(ftexi,1,[]));
end
fclose(fid);

