function write_obj2(filename, vertex, vertex_n, vertex_t, face, face_vt, face_vn, obj_name, mtlname)

% write_off - write a mesh to an OBJ file
%
%   write_obj(filename, vertex, face, options)
%
%   vertex must be of size [n,3]
%   face must be of size [p,3]
%
%   Copyright (c) 2004 Gabriel Peyr
% 
% Modified by Richard, Tai-Chiu Hsung
% Dec 20, 2013.
% 
if nargin<4
    options.null = 0;
end

if size(vertex,2)~=3
    vertex=vertex';
end
if size(vertex,2)~=3
    error('vertex does not have the correct format.');
end
if size(vertex_n,2)~=3
    vertex_n=vertex_n';
end
if size(vertex_n,2)~=3
    error('vertex_n does not have the correct format.');
end
if size(vertex_t,2)~=2
    vertex_t=vertex_t';
end
% if size(vertex_t,2)~=2
%     error('vertex_t does not have the correct format.');
% end

if size(face,2)~=3
    face=face';
end
if size(face,2)~=3
    error('face does not have the correct format.');
end
if size(face_vt,2)~=3
    face_vt=face_vt';
end
% if size(face_vt,2)~=3
%     error('face_vt does not have the correct format.');
% end
if size(face_vn,2)~=3
    face_vn=face_vn';
end
if size(face_vn,2)~=3
    error('face_vn does not have the correct format.');
end


fid = fopen(filename,'wt');
if( fid==-1 )
    error('Can''t open the file.');
    return;
end

fprintf(fid, '# write_obj2 Richard Tai-Chiu, Hsung\n');

fprintf(fid, '\nmtllib %s.mtl\n\n', obj_name);

% vertex position
fprintf(fid, '# %d vertex\n', size(vertex,1));
fprintf(fid, 'v %f %f %f\n', vertex');
fprintf(fid, 'vn %f %f %f\n', vertex_n');
if size(vertex_t,2)==2
    fprintf(fid, 'vt %f %f\n', vertex_t');
else
    disp('vertex_t does not have the correct format, skip write vertex texture.');    
end


% obj_name
fprintf(fid, '\ng %s\n', obj_name);
fprintf(fid, 'usemtl %s\n', mtlname);
fprintf(fid, 's\n\n', obj_name);
% whos face*
if size(face_vt,2)==3
    face_all = [ face(:,1), face_vt(:,1), face_vn(:,1),...
                face(:,2), face_vt(:,2), face_vn(:,2),...
                face(:,3), face_vt(:,3), face_vn(:,3) ];
    fprintf(fid, 'f %d/%d/%d %d/%d/%d %d/%d/%d\n', face_all');
else
    face_all = [ face(:,1), face_vn(:,1),...
                face(:,2), face_vn(:,2),...
                face(:,3), face_vn(:,3) ];
    fprintf(fid, 'f %d//%d %d//%d %d//%d\n', face_all');
end



fclose(fid);
